--------------------------------------------------------------------------------
-- Hero
--------------------------------------------------------------------------------

HERO_ATTACKING = 1
HERO_LOOKINGFORTARGET = 2
HERO_GOINGTOTARGET = 3
HERO_GOINGHOME = 4
HERO_PICKUPITEM = 5

Hero = {}

function Hero:new() 
	local o = {}
	setmetatable(o, self)
	self.__index = self
	
	Hero:construct(o)

	return o
end

function Hero:construct(o)
	
	o.x = -1500
	o.y = ground
	
	o.heading = 1

	o.curranim = 'idle'
	
	o.target = nil
	o.targetType = ''
	
	o.state = HERO_LOOKINGFORTARGET
	
	o.time = 0.0
	o.waitTime = 80.0
	
	local animScale = 4.0
	
	local animations = {}
	
	animations['idle'] = loadAnimation("anim/hero/hero_attack_0", 94, 81, 1, 10, animScale, true)
	animations['walk'] 	= loadAnimation("anim/hero/hero_run_0", 94, 81, 8, 10, animScale, true)
	animations['cut'] 	= loadAnimation("anim/hero/hero_attack_0", 94, 81, 7, 12, animScale, false)
	animations['pickup'] 	= loadAnimation("anim/hero/hero_pickup_0", 94, 81, 4, 10, animScale, false)
	animations['stand'] 	= loadAnimation("anim/hero/hero_stand_0", 94, 81, 4, 10, animScale, false)
	
	o.anims = animations

	o.attackedThisLoop = false
	
	o.playingWalkSound = false
	o.currsound = nil
	
	
end

function Hero:update(dt)

	self.time = self.time + dt
	if self.time < self.waitTime then
		return
	end

	local s = self.state

	local movement = 0
	
	local prevanim = self.curranim
	
	if s == HERO_GOINGHOME then
		
		
		-- Find a target
		local nearest = 1000000
		local nz = nil
		--for i = 1, NumZombies do
		for i, zom in ipairs(zombies) do
			--local z = zombies[i]
			local diff = math.abs(zom.x-self.x)
			
			if diff < nearest and not zom.dying then
				nz = zom
				nearest = diff
			end
		end
		
		
		if nearest < 400 then
			self.target = nz
			self.targetType = 'zombie'
			self.state = HERO_GOINGTOTARGET
			return
		end
		
		-- Failing to find a zombie,
		-- find loot to pick up instead
		nearest = 1000000
		local ni = nil
		local foundLoot = false
		
		for i, item in ipairs(items) do
			local diff = math.abs(item.x-self.x)
			if diff < nearest and item.alive then
				ni = item
				nearest = diff
				foundLoot = true
			end
		end
	
		if foundLoot then
			--print("Found loot")
			self.target = ni
			self.targetType = 'item'
			self.state = HERO_GOINGTOTARGET
			return
		end
		
		
		self.heading = 1
		self.x = self.x + self.heading*200*dt
		self.curranim = 'walk'
		
	end
	
	if s == HERO_LOOKINGFORTARGET then
		self.state = HERO_GOINGHOME
	end
	
	if s == HERO_GOINGTOTARGET then
	
		local diff = self.target.x-self.x
		
		if diff > 0 then 
			movement = 1
		else
			movement = -1
		end
		self.heading = movement
		
		if math.abs(diff) > 80 then
			
			self.x = self.x + self.heading*200*dt
			self.curranim = 'walk'
		else
			
			if self.targetType == 'zombie' then
			
				if self.target.health > 0 then
					self:attack(self.target)
				else
					self.state = HERO_LOOKINGFORTARGET
				end
			elseif self.targetType == 'item' then
				self:pickupItem(target)
			end
			
		end
	end
	
	
	if self.state == HERO_PICKUPITEM then
		
		if self.anims[self.curranim].finished then
			
			self.curranim = 'stand'
			self.anims[self.curranim]:play()
			self.state = HERO_STAND
			self.target.alive = false
		end
		
	end
	
	if self.state == HERO_STAND then
		if self.anims[self.curranim].finished then
			self.state = HERO_LOOKINGFORTARGET
		end
	end
	
	
	
	if s == HERO_ATTACKING then
		
		
		if self.anims[self.curranim].frameno == 6  and self.attackedThisLoop == false then
			
			self.target:takeDamage(1)
			playRandomSound("swing", VolumeSwing, self)
			self.attackedThisLoop = true
		end
		
		if self.anims[self.curranim].finished == true then			
			
			if self.target.health > 0 then
				self.state = HERO_GOINGTOTARGET
			else
				self.state = HERO_LOOKINGFORTARGET
			end
		end
	end
	
	
	local startedwalking = (not self.prevanim == 'walk') and self.curranim == 'walk'
	local stoppedwalking = self.prevanim == 'walk' and (not self.curranim == 'walk')
	
	if not self.currsound == nil then
		playingWalkSound = not self.currsound:isStopped()
		
	end
	
	if self.curranim == 'walk' then
		playRandomSound("herowalk", VolumeWalk, self)
	end
	
	self.anims[self.curranim]:update(dt)
	
	
end

function Hero:takeDamage(amount)
	
end

function Hero:attack(target)
	self.curranim = 'cut'
	self.anims[self.curranim]:play()
	--self.attacking = true
	self.state = HERO_ATTACKING
	self.attackedThisLoop = false
	
	--target:takeDamage(1)
		
end

function Hero:pickupItem(item)
	self.curranim = 'pickup'
	self.anims[self.curranim]:play()
	self.state = HERO_PICKUPITEM
	self.pickedupThisLoop = false
end

function Hero:draw()
	--self.anims[self.curranim]:draw(self.x - camerax, self.y - cameray, self.heading)
	local cam = Camera
	self.anims[self.curranim]:draw( cam.zoom * (self.x - cam.x) + cam.focusx, cam.zoom * (self.y - cam.y) + cam.focusy, self.heading, cam.zoom)
end