--------------------------------------------------------------------------------
-- MemoryBuffer
--------------------------------------------------------------------------------
			
MemoryBuffer = {}
			
function MemoryBuffer:new(seconds, framerate) 
	local o = {}
	setmetatable(o, self)
	self.__index = self
	
	o.frames = {}
	o.numframes = seconds * framerate
	o.nrframes = 0
	
	o.framerate = framerate
	o.frametime = 1.0 / framerate
	
	o.record = false
	
	o.time = 0.0
	o.time2 = 0.0
	o.a = 0
	
	o.currindex = 1
	
	return o
end

function MemoryBuffer:update(dt)
	self.time = self.time + dt
	self.time2 = self.time2 + dt
	self.a = 0.5 + 0.5*math.sin(self.time2)
	
	if self.time > self.frametime then
		self.record = true
		self.time  = self.time - self.frametime
	end
end

function MemoryBuffer:capture()
	
	if self.record then
		self:recordFrame()
		self.record = false
	end
end

function MemoryBuffer:recordFrame()
	--love.graphics.setRenderTarget(recordFB)
	
	local r,g,b,a = love.graphics.getColor()
	local cmode = love.graphics.getColorMode()
	local blend = love.graphics.getBlendMode()
	
	
		
	
	love.graphics.push()
		love.graphics.scale(memScale, memScale)
		
		love.graphics.setRenderTarget(testFB)
		drawScene()
		
		love.graphics.setRenderTarget(recordFB)
		
		love.graphics.draw(cloudsa, 400+ math.sin(self.time2)*20, 300, math.sin(self.time2)*0.1, 1, 1, 400, 300)
		
		love.graphics.setBlendMode('multiplicative')
		
		love.graphics.draw(testFB, 0, 0, 0, 2, 2, 0, 0)
		
	love.graphics.pop()
	
	love.graphics.setColor(r,g,b,a)
	love.graphics.setColorMode(cmode)
	love.graphics.setBlendMode(blend)
	
	love.graphics.setRenderTarget()
	
	
	local imgd = recordFB:getImageData()
	self.last = love.graphics.newImage(imgd)
	self.frames[self.currindex] = imgd
	--print("mem buffer: currindex = " .. self.currindex)
	
	self.currindex = self.currindex + 1
	if self.currindex > self.numframes then
		self.currindex = 1
	end
	
	self.nrframes = math.min(self.nrframes + 1, self.numframes)
	
	
end

function MemoryBuffer:getMemory()

	local memory = Memory:new(self.framerate)
	
	local index = math.max(self.currindex - 1, 1)
	
	for i = 1, self.nrframes do
		index = index + 1
		if index > self.nrframes then
			index = 1
		end
		
		local imgd = self.frames[index]
		memory:addFrame(imgd)
	end
	
	return memory
end

function sepiaTone(x, y, r, g, b, a)
    
	r = math.min(r*0.393 + g*0.769 + b*0.189, 255)
	g = math.min(r*0.349 + g*0.686 + b*0.168, 255)
	b = math.min(r*0.272 + g*0.534 + b*0.131, 255)
	
    return r, g, b, a
end




--------------------------------------------------------------------------------
-- Memory
--------------------------------------------------------------------------------

Memory = {}
			
function Memory:new(framerate) 
	local o = {}
	setmetatable(o, self)
	self.__index = self
	
	local animation = Animation:new()
	
	animation.loop = false
	animation.scale = 2.0
	animation.framerate = framerate
	animation.sizex = memWidth
	animation.sizey = memHeight
	
	animation.framerate = 4
	
	animation.fade = true
	animation.fadetime = 0.25
	
	o.anim = animation
	
	o.x = 350
	o.y = 500
	
	o.alpha = 0.0
	o.timer = 0.0
	o.fadeout = false
	
	return o
end

function Memory:addFrame(imgd)
	local img = love.graphics.newImage(imgd)
	self.anim:addFrame(img)
end

function Memory:update(dt)
	--print("MEM:update, numframes= " .. self.anim.numframes .. ", frameno= " .. self.anim.frameno)
	self.anim:update(dt)
	
	self.timer = self.timer + dt
	
	self.alpha = math.min( self.timer / 1.0, 1.0)
	
	if self.fadeout then
		self.alpha = math.max(1.0 - self.alpha, 0.0)
	end
	
	if self.anim.finished then
		-- at the beginning of the fade out, reset the timer
		if self.fadeout == false then
			self.timer = 0.0
		end
		
		self.fadeout = true
	end
end

function Memory:isFinished()
	return self.anim.finished
end

function Memory:finishedDrawing()
	return self.fadeout == true and self.alpha == 0.0
end

function Memory:reset()
	self.anim:play()
	self.timer = 0.0
	self.fadeout = false
	--print("resetting memory-----------------")
	--print("num frames " .. self.anim.numframes .. ", current frame: " .. self.anim.frameno)
end

function Memory:draw()
	
	local r,g,b,a = love.graphics.getColor()
	
	--love.graphics.setColor(255*0.7, 255*0.6, 255*0.1, 255 * self.alpha)
	love.graphics.setColor(255*0.4, 255*0.7, 255*0.9, 255 * self.alpha)
	self.anim:draw(self.x, self.y, 1)
	
	love.graphics.setColor(r,g,b,a)
end

function wa()
	--print(12)
end









