--------------------------------------------------------------------------------
-- Player
--------------------------------------------------------------------------------

Player = {}

function Player:new() 
	local o = {}
	setmetatable(o, self)
	self.__index = self
	
	Player:construct(o)
	
	return o
end

function Player:construct(o)
	
	o.x = 0
	o.y = 0
	o.heading = 1
	
	o.health = 3
	o.takingdamage = false
	o.dying = false
	o.deathanim = 'die'
	
	o.attacking = false
	o.startingtoeat = false
	o.eating = false
	o.endingeat = false
	
	local animScale = 4.0
	
	local animations = {}
	
	animations['walk'] 			= loadAnimation("anim/player/player_walk_small", 126, 68, 8, 5, animScale, true)
	--animations['walk'].fade = true
	--animations['walkattack'] 	= loadAnimation("anim/zombie/zombie_chase_0", 126, 68, 8, 10, animScale, true)
	animations['idle'] 			= loadAnimation("anim/player/player_idle_small", 126, 68, 4, 5, animScale, true)
	animations['begineat']		= loadAnimation("anim/player/player_crawl_small", 126, 68, 9, 10, animScale, false)
	animations['endeat']		= loadAnimation("anim/player/player_stand_small", 126, 68, 9, 10, animScale, false)
	animations['eat'] 			= loadAnimation("anim/player/player_eat_small", 126, 68, 5, 10, animScale, true)
	animations['hurt'] 			= loadAnimation("anim/player/player_hit_small", 126, 68, 2, 10, animScale, false)
	animations['attack'] 		= loadAnimation("anim/player/player_attack_small", 126, 68, 7, 10, animScale, false)
	animations['die'] 			= loadAnimation("anim/player/player_die_small", 126, 68, 7, 10, animScale, false)
	animations['dieclosed']		= loadAnimation("anim/player/player_die_closed_small", 126, 68, 1, 10, animScale, false)
	
	o.anims = animations
	o.curranim = 'idle'
	
	o.eatduration = 0.0
	o.capturedeating = false
	o.deadduration = 0.0
	o.captureddying = false
	
	o.time = 0.0
	
	o.nextcapture = math.random(o.time + 5.0, o.time + 30)
	o.capture = true
	o.numcaptures = 0
	
end

function Player:update(dt)

	self.time = self.time + dt
	--
	self.anims[self.curranim]:update(dt)
	
	if self.capture and self.numcaptures < 2 and self.time >= self.nextcapture then
		captureMemory()
		self.nextcapture = math.random(self.time + 5.0, self.time + 30)
		self.numcaptures = self.numcaptures + 1
	end
	
	if self.dying then
		
		if self.x < 400 then self.heading = -1 end
		if self.x > 1600-400 then self.heading = 1 end
		
		self.curranim = self.deathanim
		self.deadduration = self.deadduration + dt
		
		self.capture = false
		
		if not self.captureddying then
			
			if self.deadduration > 2.0 then
				captureMemory()
				self.captureddying = true
			end			
		end
		
		return
	end
	
	
	if self.takingdamage then
		self.takingdamage = not self.anims[self.curranim].finished
		return
	end
	
	if self.attacking then
		self.attacking = not self.anims[self.curranim].finished
		return
	end
	
	if self.startingtoeat then
		playRandomSound("eat", VolumeEat, self)
		if self.anims['begineat'].finished then
			self.curranim = 'eat'
			self.anims[self.curranim]:play()
			self.eating = true
			self.startingtoeat = false
		end
		return
	end
	
	if self.eating then
		
		if not self.capturedeating then
			self.eatduration = self.eatduration + dt
			
			if self.eatduration > 2.5 then				
				captureMemory()
				self.capturedeating = true
			end	
		end
		
		if love.keyboard.isDown('right') or love.keyboard.isDown('left') or love.keyboard.isDown('up') or love.keyboard.isDown(' ') then
			self.curranim = 'endeat'
			self.anims[self.curranim]:play()
			self.endingeat = true
			self.eating = false
		end
		return
	end
	
	if self.endingeat then
		self.endingeat = not self.anims[self.curranim].finished
		self.eatduration = 0.0
		return
	end
	
	
	
	local movement = 0

	if love.keyboard.isDown('left') then
		movement = movement - 1
		self.heading = -1
		self.curranim = 'walk'
	end
	
	if love.keyboard.isDown('right') then
		movement = movement + 1
		self.heading = 1
		self.curranim = 'walk'
	end
	
	if movement == 0 then
		self.curranim = 'idle'
	 end
	
	if love.keyboard.isDown(' ') then
		self.curranim = 'attack'
		self.anims[self.curranim]:play()
		self.attacking = true
	end
	
	if love.keyboard.isDown('down') then
	
		-- Start to eat, if in range
		local cmid = corpsestart + (corpseend - corpsestart) / 2
		local diff = self.x - cmid
		
		if math.abs(diff) < 200 then
			self.curranim = 'begineat'
			self.anims[self.curranim]:play()
			self.startingtoeat = true
			
			if diff < 0 then
				self.heading = 1
			elseif diff > 0 then
				self.heading = -1
			end
		end
	end
	
	
	
	self.x = self.x + movement*70*dt
	
	-- Constrain movement [0,1600] minus offset
	local offset = 50
	if self.x < offset then
		self.x = offset
	end
	
	if self.x > 1600-offset then
		self.x = 1600-offset
	end
	
	

end

function Player:draw()
	local cam = Camera
	--self.anims[self.curranim]:draw(self.x - camerax, self.y - cameray, self.heading)
	self.anims[self.curranim]:draw( cam.zoom * (self.x - cam.x) + cam.focusx, cam.zoom * (self.y - cam.y) + cam.focusy, -self.heading, cam.zoom)
end

function Player:takeDamage(amount)
	
	if self.dying == true then
		return
	end

	self.health = self.health - amount
	if self.health <= 0 then
		self.dying = true
		
		playRandomSound("die", VolumeDie, self)
		
		--Spawn Loot!
		--- Spawn loot at all?
		if math.random() > 0.3 then
			local anim = ''
			-- Which type?
			if math.random() > 0.5 then
				anim = 'gold'
			else
				anim = 'potion'
			end
			
			self:dropLoot(anim)
				
		end
		
		
		
	else
		self.takingdamage = true
		self.curranim = 'hurt'
		self.anims[self.curranim]:play()
		playRandomSound("hit", VolumeHit, self)
	end
	
end

function Player:dropLoot(name)
	local item = Item:new()
	item.x = self.x
	item.y = self.y
	item.curranim = name
	table.insert(items, item)
end

function Player:closeEyes()
	self.deathanim = "dieclosed"
end
