--------------------------------------------------------------------------------
-- Sound
--------------------------------------------------------------------------------

RandomSoundPlayer = {}

function RandomSoundPlayer:new()
	local o = {}
	setmetatable(o, self)
	self.__index = self

	o.sounds = {}
	o.volume = 1.0
	o.playing = false
	o.currsound = nil

	return o
end

function RandomSoundPlayer:add(sounds)

	for i,s in ipairs(sounds) do
		table.insert(self.sounds, s)	
	end	

	self.currsound = self.sounds[1]
	
end

function RandomSoundPlayer:play()
	if not playing then
		local rs = math.random(#self.sounds)
		self.currsound = self.sounds[rs]
		self.currsound:setVolume(self.volume)
		self.currsound:play()
		self.playing = true
	end
end

function RandomSoundPlayer:setVolume(volume)
	self.volume = volume
end





Sound = {}

function Sound:new(source)
	local o = {}
	setmetatable(o, self)
	self.__index = self
	
	
	o.source = source
	
	return o
end

function Sound:play(loop)
	if loop then self.source:setLooping(loop) end
	--love.audio.stop(self.source)
	
	if self.source:isStopped() then
		love.audio.play(self.source)
	end
end

function Sound:playIfDone(loop)
	if loop then self.source:setLooping(loop) end
	love.audio.play(self.source)
end

function Sound:stop()
	love.audio.stop(self.source)
end

function Sound:setVolume(v)
	self.source:setVolume(v)
end



function loadSound(path, ismusic)
	local sound_type = "static"
	if ismusic then sound_type = "stream" end
	local source = love.audio.newSource(path, sound_type)
	source:setVolume(1.0)
	
	return Sound:new(source)
end


function playRandomSound(name, volume, object, loop)
	
	local attenuation = 1.0
	if object then
		attenuation = 1.0 - math.min( math.abs(player.x - object.x) / PlayerSoundRange, 1.0 )	
	end
	
	

	local ss = sounds[name]
	local s = ss[math.random(#ss)]
	s:setVolume(volume * attenuation)
	
	s:play()
	
	return s
end



