--------------------------------------------------------------------------------
-- Zombie
--------------------------------------------------------------------------------
Zombie = {}

function Zombie:new() 
	local o = {}
	setmetatable(o, self)
	self.__index = self
	
	Zombie:construct(o)
	
	return o
end



function Zombie:construct(o)
	
	o.x = 0
	o.y = 0
	o.origx = 0
	
	o.velx = 0
	o.vely = 0
	
	o.heading = 1
	o.walkgoalx = o.x
	
	o.state = 0
	o.beginstate = false
	o.endstate = false
	
	o.idleduration = 0.0
	o.idletimer = 0.0
	
	o.walktargetx = 0
	o.haswalktarget = false
	
	o.atcorpse = false
	o.eatduration = 0.0
	o.eattimer = 0.0
	o.startingtoeat = false
	o.endingeat = false
	
	o.health = 3
	o.takingdamage = false
	o.dying = false
	
	o.startattacking = false
	o.startrun = true
	
	local animScale = 4.0
	
	
	o.curranim = ''
	
	local animations = {}
	
	animations['walk'] 			= loadAnimation("anim/zombie/zombie_walk_0", 126, 68, 8, 7, animScale, true)
	animations['walkattack'] 	= loadAnimation("anim/zombie/zombie_chase_0", 126, 68, 8, 10, animScale, true)
	animations['stand'] 		= loadAnimation("anim/zombie/zombie_idle_0", 126, 68, 4, 2, animScale, true)
	animations['begineat']		= loadAnimation("anim/zombie/zombie_crawl_0", 126, 68, 9, 10, animScale, false)
	animations['endeat']		= loadAnimation("anim/zombie/zombie_stand_0", 126, 68, 9, 10, animScale, false)
	animations['eat'] 			= loadAnimation("anim/zombie/zombie_eat_0", 126, 68, 5, 10, animScale, true)
	animations['hurt'] 			= loadAnimation("anim/zombie/zombie_hit_0", 126, 68, 2, 10, animScale, false)
	animations['attack'] 		= loadAnimation("anim/zombie/zombie_attack_0", 126, 68, 7, 10, animScale, false)
	animations['die'] 			= loadAnimation("anim/zombie/zombie_die_0", 126, 68, 7, 10, animScale, false)
	
	o.anims = animations
	
	
end

function Zombie:update(dt)

	if self.curranim == '' then
		self.curranim = 'stand'
	end
	
	if self.dying then
		self.curranim = 'die'
		self.anims[self.curranim]:update(dt)
		return
	end
	
	if self.takingdamage == true then
		
		if self.anims[self.curranim].finished == true then
			self.curranim = 'stand'
			self.takingdamage = false
			
		else
			self.anims[self.curranim]:update(dt)
			return
		end
	end
	
	-- States
	ZOMBIE_CHOOSE = 0
	ZOMBIE_IDLE = 1
	ZOMBIE_WALK = 2
	ZOMBIE_EAT = 3
	ZOMBIE_GOINGTOTARGET = 4
	ZOMBIE_ATTACKING = 5
	ZOMBIE_BEGIN_EAT = 6
	ZOMBIE_END_EAT = 7
	ZOMBIE_EAT_WALK = 8
	ZOMBIE_EAT_ACTUAL = 9
	
	local hasHeroTarget = false
	
	local HeroTargetDistance = 400
	
	
	-- First of all, see if the hero is in range. If so, attack him
	
	local distanceToHero = math.abs(self.x-hero.x)
	local heroInRange = distanceToHero < HeroTargetDistance
	
	if heroInRange then
		if self.state == ZOMBIE_EAT_ACTUAL or self.state == ZOMBIE_END_EAT or self.state == ZOMBIE_BEGIN_EAT or self.state == ZOMBIE_ATTACKING then
			--self.state = ZOMBIE_END_EAT
		else
			if self.startrun then
				playRandomSound("spot", VolumeSpot, self)
				self.startrun = false
			end
			self.state = ZOMBIE_GOINGTOTARGET
		end
	else
		if not (self.state == ZOMBIE_IDLE or self.state == ZOMBIE_WALK or self.state == ZOMBIE_EAT or self.state == ZOMBIE_BEGIN_EAT or self.state == ZOMBIE_END_EAT or self.state == ZOMBIE_EAT_ACTUAL) then
			self.state = ZOMBIE_CHOOSE
		end
	end
	
	-- Head to the hero
	if self.state == ZOMBIE_GOINGTOTARGET then
		if distanceToHero < 70 then
			self.haswalktarget = false
			
			self.state = ZOMBIE_ATTACKING
			self.startattacking = true
	
			hero:takeDamage(1)
		else
			hasHeroTarget = true
			self.walktargetx = hero.x
			self.haswalktarget = true
		end
	end
	
	-- Attack
	if self.state == ZOMBIE_ATTACKING then
		
		if self.startattacking then
			self.curranim = 'attack'
			self.anims[self.curranim]:play()
			self.startattacking = false
		else	
			self.curranim = 'attack'
			if self.anims[self.curranim].finished then
				self.state = ZOMBIE_GOINGTOTARGET
			end
		end
	end
	
	-- Decide what to do next
	if self.state == ZOMBIE_CHOOSE then
		--print("Choosing state...")
		
		if math.abs(self.x - (corpsestart + corpseend)/2) > 300 or numZombiesEating >= 1 then
			self.state = math.random(1,2)
		else
			self.state = math.random(1,3)
		end
		
		self.beginstate = true
	end
	
	-- Idle
	if 	self.state == ZOMBIE_IDLE then 
		if self.beginstate then
			
			self.curranim = 'stand'
			self.idleduration = math.random(1.0, 5.0)
			self.idletimer = 0.0
			self.beginstate = false
			
			--print("IDLE: " .. self.idleduration)
		else
			self.idletimer = self.idletimer + dt
			if self.idletimer >= self.idleduration then
				self.state = ZOMBIE_CHOOSE
			end
		end
	end
		
	-- Walk
	if self.state == ZOMBIE_WALK then 
		if self.beginstate then
			
			--self.walktargetx = math.random(20, love.graphics.getWidth()-20)
			self.walktargetx = math.random(self.origx - 200, self.origx + 200)
			self.haswalktarget = true
			self.beginstate = false
			
			--print("WALK: " .. self.walktargetx)
		else
			if self.haswalktarget == false then
				self.state = ZOMBIE_CHOOSE
			end
		end
	end
		
	-- Eat
	if self.state == ZOMBIE_EAT then
		if self.beginstate then
			self.beginstate = false
			-- Find a nice spot on the corpse
			
			numZombiesEating = numZombiesEating + 1
			
			self.walktargetx = math.random(corpsestart, corpseend)
			
			local cmid = corpsestart + (corpseend - corpsestart) / 2
			local dir = self.walktargetx - self.x
			--print("" .. corpsestart .. " " ..corpseend .. " " .. cmid .. " " .. dir)
			
			if dir < 0 and self.walktargetx < cmid then
				self.walktargetx = cmid
			elseif dir > 0 and self.walktargetx > cmid then
				self.walktargetx = cmid
			end
			
			self.haswalktarget = true
			self.eatduration = math.random(4.0, 7.0)
			self.eattimer = 0.0
			self.startingtoeat = true
			
			--print("EAT: " .. self.walktargetx .. ", " .. self.eatduration)
		else
			
		
			-- We're there, start eating
			if self.haswalktarget == false then
			
				self.state = ZOMBIE_BEGIN_EAT
				self.startingtoeat = true
				
				--[[
				self.curranim = 'eat'
				
				self.eattimer = self.eattimer + dt
				if self.eattimer >= self.eatduration then
					self.state = ZOMBIE_CHOOSE
				end
				--]]
			end
			
		end
	end
	
	if self.state == ZOMBIE_BEGIN_EAT then
		if self.startingtoeat then
			self.curranim = 'begineat'
			self.anims[self.curranim]:play()
			
			playRandomSound("eat", VolumeEat)
			
			
			self.startingtoeat = false
		else
			if self.anims[self.curranim].finished then
				self.state = ZOMBIE_EAT_ACTUAL
			end
		end
	end
	
	if self.state == ZOMBIE_EAT_ACTUAL then
		self.curranim = 'eat'
			
		self.eattimer = self.eattimer + dt
		if self.eattimer >= self.eatduration or heroInRange then
			self.state = ZOMBIE_END_EAT
			self.endingeat = true
		end
	end
	
	if self.state == ZOMBIE_END_EAT then
		if self.endingeat then
			self.curranim = 'endeat'
			self.anims[self.curranim]:play()
			numZombiesEating = numZombiesEating - 1
			self.endingeat = false
		else
			if self.anims[self.curranim].finished then
				if heroInRange then
					self.state = ZOMBIE_GOINGTOTARGET
				else
					self.state = ZOMBIE_CHOOSE
				end
			end
		end
		
	end
	
	
	
	if self.haswalktarget then
			if hasHeroTarget then
				self.curranim = 'walkattack'
			else
				self.curranim = 'walk'
			end
			
			local targetDist = self.walktargetx - self.x
			
			if targetDist > 0 then
				self.heading = 1
			else
				self.heading = -1
			end
			
			-- Walk towards the target
			self.x = self.x + self.heading*100*dt
			
			if math.abs(targetDist) < 5 then
				self.haswalktarget = false
			end
	end

	self.anims[self.curranim]:update(dt)
end

function Zombie:draw()
	local c = Camera
	self.anims[self.curranim]:draw(c.zoom*(self.x - c.x) + c.focusx, c.zoom*(self.y - c.y) + c.focusy, -self.heading, c.zoom)
end

function Zombie:takeDamage(amount)
	
	if self.dying == true then
		return
	end

	self.health = self.health - amount
	if self.health <= 0 then
		self.dying = true
		
		playRandomSound("die", VolumeDie, self)
		
		--Spawn Loot!
		--- Spawn loot at all?
		if math.random() > 0.3 then
			local anim = ''
			-- Which type?
			if math.random() > 0.5 then
				anim = 'gold'
			else
				anim = 'potion'
			end
			
			self:dropLoot(anim)
				
		end
		
		
		
	else
		self.takingdamage = true
		self.curranim = 'hurt'
		self.anims[self.curranim]:play()
		playRandomSound("hit", VolumeHit, self)
	end
	
end

function Zombie:dropLoot(name)
	local item = Item:new()
	item.x = self.x
	item.y = self.y
	item.curranim = name
	table.insert(items, item)
end