
require "Animation"
require "Zombie"
require "Hero"
require "Player"
require "Item"
require "Memory"
require "Sound"

Debug = false

--------------------------------------------------------------------------------
-- LOVE Functions
--------------------------------------------------------------------------------

function love.load()
	
	--print("Loading!")
	
	local w = love.graphics:getWidth()
	local h = love.graphics:getHeight()
	
	local halfw = w/2.0
	local halfh = h/2.0
	
	timeacc = 0
	
	font = love.graphics.newFont("data/consola.ttf", 20)
	love.graphics.setFont(font)
	
	Camera = {}
	Camera.x = halfw
	Camera.y = halfh
	Camera.zoom = 1.0
	Camera.focusx = halfw
	Camera.focusy = halfh
	
	zoom = 1.0
	
	-- Init some utility vars
	ground = 436
	
	background = loadImage("anim/env_400x150.png")
	
	black = loadImage("black.png")
	title = loadImage("title.png")
	title_text = loadImage("title_text.png")
	instructions = loadImage("instructions.png")
	bgAlpha = 0.0
	titleAlpha = 0.0
	textAlpha = 0.0
	gtime = 0.0

	targetx = -1000
	
	corpse = loadAnimation("anim/corpse/corpse", 126, 68, 8, 5, 4, true)
	
	corpsestart = 400
	corpseend = 600
	
	-- Nice color!
	love.graphics.setBackgroundColor(0.4*255, 0.5*255, 0.6*255)
	
	
	NumZombies = 7
	NumPreZombies = 3
	zombies = {}
	for i = 1, NumPreZombies do
		local z = Zombie:new()
		z.x = math.random(-1000, -100)
		z.origx = z.x
		z.y = ground
		table.insert(zombies, z)
	end
	
	for i = 1, NumZombies - NumPreZombies do
		local z = Zombie:new()
		z.x = math.random(0, 1600)
		z.origx = z.x
		z.y = ground
		table.insert(zombies, z)
	end
	
	numZombiesEating = 0
	
	player = Player:new()
	player.x = 900
	player.y = ground
	table.insert(zombies, player)
	
	
	
	hero = Hero:new()
	
	items = {}
	
	memScale = 0.5
	memWidth = love.graphics:getWidth() * memScale
	memHeight = love.graphics:getHeight() * memScale
	recordFB = love.graphics.newFramebuffer(memWidth, memHeight)
	testFB = love.graphics.newFramebuffer(memWidth, memHeight)
	
	cloudsa = love.graphics.newImage("malpha.png")
	
	membuffer = MemoryBuffer:new(5, 10)
	memories = {}
	numMemories = 0
	
	
	
	zoomtimeacc = 0.0
	
	playerdeadtimer = 0.0
	GameOverScreenTime = 5.0
	
	canRestart = false
	showGameOverScreen = false
	showMemories = false
	newmem = true
	memindex = 1
	
	
	sounds = {}
	
	sounds["herowalk"] = { loadSound("sounds/armor1.ogg") }
	sounds["moan"] = { loadSound("sounds/moan1.ogg"),loadSound("sounds/moan2.ogg") }
	sounds["spot"] = { loadSound("sounds/spot1.ogg"), loadSound("sounds/spot2.ogg"), loadSound("sounds/spot3.ogg") }
	sounds["eat"] = { loadSound("sounds/eat1.ogg"), loadSound("sounds/eat2.ogg"), loadSound("sounds/eat3.ogg") }
	sounds["die"] = { loadSound("sounds/die1.ogg"), loadSound("sounds/die2.ogg"), loadSound("sounds/die3.ogg") }
	sounds["swing"] = { loadSound("sounds/swoosh.ogg") }
	sounds["hit"] = { loadSound("sounds/punch.ogg") }
	sounds["crunch"] = { loadSound("sounds/crunch.ogg") }
	sounds["drops"] = { loadSound("sounds/water_drops.ogg") }
	sounds["wind"] = { loadSound("sounds/wind.ogg", true) }
	sounds["music"] = { loadSound("sounds/music.ogg", true) }
	
	sounds["drops"][1]:setVolume(0.4)
	sounds["drops"][1]:play(true)
	
	sounds["wind"][1]:setVolume(0.6)
	sounds["wind"][1]:play(true)
	
	musicplaying = false
	
	
	VolumeMaster = 1.0
	VolumeMusic = 0.4
	VolumeSwing = 0.1
	VolumeHit = 0.2
	VolumeSpot = 0.6
	VolumeEat = 0.2
	VolumeDie = 0.7
	VolumeWalk = 0.4
	
	PlayerSoundRange = 2000.0

	
end

function love.update(dt)

	if player.dying and player.deadduration > 2.0 and not musicplaying then
		sounds["music"][1]:setVolume(VolumeMusic)
		sounds["music"][1]:play(false)
		musicplaying = true
	end
	
	timeacc = timeacc + dt

	-- Update the chase target
	targetx = love.mouse.getX()
	
	-- Update all the actors
	
	for i,z in ipairs(zombies) do
		z:update(dt)
	end
	
	for i,t in ipairs(items) do
		t:update(dt)
	end
	
	hero:update(dt)
	
	corpse:update(dt)
	
	membuffer:update(dt)
	
	updateMemories(dt)
	
	
	local w2 = love.graphics.getWidth() / 2.0
	-- Focus camera on the player
	Camera.x = player.x
	--Camera.x = hero.x
	-- But constrain its movements
	if Camera.x < w2 then Camera.x = w2 end
	if Camera.x > 1600-w2 then Camera.x = 1600-w2 end
	
	updateZoom(dt)
	
	
	if showGameOverScreen then
		updateGameOverScreen(dt)
	end
	
	
	
end

function love.draw()
	
	drawScene()
	
	membuffer:capture()
	
	drawMemories()
	
	if Debug then
		local fps = love.timer.getFPS()
		love.graphics.print(string.format("FPS: %i\nmS:  %.2f\nTime: %.1f", fps, 1000.0 / fps, timeacc), 650, 0)
	end
	
end

function drawScene() 
	
	local w = love.graphics:getWidth()
	local h = love.graphics:getHeight()
	
	local halfw = w/2.0
	local halfh = h/2.0
	
	-- Draw background
	love.graphics.draw(background, Camera.zoom*(0 - Camera.x) + Camera.focusx, Camera.zoom*(0-Camera.y) + Camera.focusy, 0, 4*Camera.zoom, 4*Camera.zoom, 0,0)
	
	local corpsex = (corpsestart + corpseend) / 2
	local corpsey = ground
	corpse:draw(Camera.zoom * (corpsex - Camera.x) + Camera.focusx, Camera.zoom * (corpsey - Camera.y) + Camera.focusy, 1, Camera.zoom)

	-- Draw all actors
	for i,z in ipairs(zombies) do
		z:draw()
	end	
	
	for i,t in ipairs(items) do
		t:draw()
	end
	
	hero:draw()
	
	-- Fade in from black right at the start
	local r,g,b,a = love.graphics.getColor()
	local ba = math.max(1.0 - math.min(timeacc / 1.0, 1.0), 0.0)
	love.graphics.setColor(255, 255, 255, 255 * ba)
	love.graphics.draw(black, 0, 0, 0, 4, 4, 0)
	love.graphics.setColor(r,g,b,a)
	
	drawGameOverScreen()
	
	drawInstructions();
	
	
end





function updateZoom(dt)

	if not player.dying or player.deadduration < 3.0 then
		return
	end
	
	local ZoomMin = 1.0
	local ZoomMax = 2.0
	local ZoomTime = 5.0
	
	zoomtimeacc = zoomtimeacc + dt
	local a = math.min(zoomtimeacc / ZoomTime, 1.0)
	a = (1.0 - math.cos(a*math.pi))/2.0
	
	Camera.zoom = ZoomMax*a + ZoomMin*(1.0 - a)
	
	
	local halfwidth = love.graphics.getWidth() / 2
	
	local screenx = math.min(player.x, halfwidth)
	
	if player.x > Camera.x then
		screenx = halfwidth + player.x - Camera.x
	end
	
	
	-- offset a bit to focus more on the head
	Camera.focusx = screenx - Camera.zoom * (player.x - 100*a*player.heading - Camera.x)
	
end


function updateMemories(dt)
	
	if player.dying and player.deadduration > 8.0 then
		showMemories = not showGameOverScreen
	end
	--showMemories = true
	
	if not showMemories then
		return
	end
	
	if numMemories <= 0 then
		return
	end
	
	local mem = memories[memindex]
	
	if newmem then
		mem:reset()
		newmem = false
	end
	
	mem:update(dt)
	
	if mem:finishedDrawing() then
		memindex = memindex + 1
		newmem = true
		
		if memindex > numMemories then
			showMemories = false
			showGameOverScreen = true
			
		end
	end
	
	
	
	
end


function drawMemories()

	if not showMemories then
		return
	end
	
	if numMemories <= 0 then
		return
	end

	memories[memindex]:draw()
end





function love.keypressed(key, unicode)
	
	if key == 'escape' then
		love.event.push('q')
	end
	
	if key == 'return' then
		if canRestart then
			reloadGame()
		end
	end

	if key == 'd' then
		Debug = not Debug
	end
	
	-- Debug stuff
	if Debug then
	
		-- if key == 'left' then
		-- 	Camera.x = Camera.x - 10
		-- end
		-- if key == 'right' then
		-- 	Camera.x = Camera.x + 10
		-- end
	
		-- if key == 'r' then
		-- 	reloadGame()
		-- end
		
		-- if key == 'm' then
			
		-- 	if musicplaying then
		-- 		sounds["music"][1]:stop()
		-- 		musicplaying = false
		-- 	else
		-- 		sounds["music"][1]:play(false)
		-- 		musicplaying = true
		-- 	end
			
			
		-- end
	
		-- if key == 'f' then
		-- 	local z = Zombie:new()
		-- 	z.x = math.random(0, love.graphics.getWidth())
		-- 	z.y = ground
			
		-- 	table.insert(zombies, z)
		-- end
		
		-- if key == 'g' then
		-- 	local m = membuffer:getMemory()
		-- 	m.x = math.random(0, love.graphics:getWidth())
		-- 	m.y = math.random(0, love.graphics:getHeight())
		-- 	table.insert(memories, m)
		-- end
		
	end
end



function love.mousepressed(x, y, button)
	if Debug then
		if button == 'l' then
			player:takeDamage(1)
		end
	end
end



function loadImage(path)
	local img = love.graphics.newImage(path)
	img:setFilter('nearest', 'nearest')
	return img
end

function drawInstructions(dt)
	
	-- Fade in
	local alpha = math.min( math.max(0, timeacc-3.0), 1.0)
	
	-- Fade out
	if timeacc > 10.0 then
		alpha = 1 - math.min(timeacc-10.0, 1.0)
	end
	
	local r,g,b,a = love.graphics.getColor()
	
	love.graphics.setColor(255, 255, 255, alpha * 255)
	love.graphics.draw(instructions, 0, 0, 0, 4, 4, 0, 0)
	love.graphics.setColor(r, g, b, a)
	
end

function updateGameOverScreen(dt)
	
	playerdeadtimer = playerdeadtimer + dt
	if playerdeadtimer < GameOverScreenTime then
		return
	end
	
	player:closeEyes()
	
	gtime = gtime + dt
	
	bgAlpha = math.min(gtime / 2.0, 1.0)
	
	if gtime > 3.0 then
		titleAlpha = math.min( (gtime-3.0) / 1.0, 1.0)
	end
	
	if gtime > 5.0 then
		textAlpha = math.min( (gtime-5.0) / 1.0, 1.0)
		canRestart = true
	end
	
end

function drawGameOverScreen()
	
	local w = love.graphics.getWidth()
	local h = love.graphics.getHeight()

	local r,g,b,a = love.graphics.getColor()

	love.graphics.setColor(255, 255, 255, bgAlpha * 255)
	love.graphics.draw(black, 0, 0, 0, 4, 4, 0, 0)
	
	love.graphics.setColor(255, 255, 255, titleAlpha * 255)
	love.graphics.draw(title, 0, 0, 0, 4, 4, 0, 0)
	
	love.graphics.setColor(255, 255, 255, textAlpha * 255)
	love.graphics.draw(title_text, 0, 0, 0, 4, 4, 0, 0)
	
	love.graphics.setColor(r,g,b,a)
end

function captureMemory()
	local m = membuffer:getMemory()
	table.insert(memories, m)
	numMemories = numMemories + 1
	--print("Captured memory.")
end

function resetState() 
	sounds["music"][1]:stop()
end

function reloadGame()
	
	resetState();
	
	love.filesystem.load("main.lua")
	love.filesystem.load("Animation.lua")
	love.filesystem.load("Zombie.lua")
	love.filesystem.load("Hero.lua")
	love.filesystem.load("Player.lua")
	love.filesystem.load("Item.lua")
	love.filesystem.load("Memory.lua")
	
	love.load()
end






